import random

import skills
import mobs
import engine

class Player:
    def __init__(self):
        self.health = 0
        self.str = 0
        self.agi = 0
        self.con = 0
        self.mag = 0
        self.wil = 0
        
        self.maxSp = 1
        self.sp = self.maxSp
        
        self.exp = 0
        self.expToLevel = 1
        self.level = 1
        self.statPoints = 0
        
        self.formExp = {}
        
        self.skills = []
        self.mobile = mobs.Mobile(0, 0, 0)
        
        self.addSkill(skills.Possess())
        self.addSkill(skills.Depossess())
     
    def addSkill(self, skill):
        for sk in self.skills:
            if sk.name == skill.name:
                return
        self.skills.append(skill)
        
    def gainExpFromMobile(self, mobile):
        exp = mobs.hazardRatings[mobile.getName()]
        self.gainExp(int(round(pow(exp, 1.5))))
        self.addFormExp(exp)
        return exp
    
    def addFormExp(self, exp):
        name = self.mobile.getName()
        
        if name == "Imp":
            return
        
        if not name in self.formExp:
            self.formExp[name] = 0
        
        self.formExp[name] += exp
        hazardRating = mobs.hazardRatings[name]
        requiredExp = hazardRating * 10
        
        while self.formExp[name] >= requiredExp:
            self.learnNewAbility()
            self.formExp[name] -= requiredExp
            
    def learnNewAbility(self):
        skills = self.mobile.skills
        unlearnedSkills = []
        
        for skill in skills:
            found = False
            for knownSkill in self.skills:
                if knownSkill.name == skill.name:
                    found = True
                    break
            
            if not found:
                unlearnedSkills.append(skill)
                
        if len(unlearnedSkills) == 0:
            return
        
        newSkill = unlearnedSkills[random.randint(0, len(unlearnedSkills) - 1)]
        self.skills.append(newSkill)
        
        engine.log("You have learned the ability %s!" % newSkill.name)
        
    def regenSp(self):
        self.sp += self.level * 0.1
        if self.sp > self.maxSp:
            self.sp = self.maxSp
         
    def gainExp(self, exp):
        self.exp += exp

        levelsGained = 0
        while self.exp >= self.expToLevel:
            levelsGained += 1
            self.level += 1
            self.maxSp += 1
            self.sp += 1
            self.statPoints += 3
            self.expToLevel += pow(self.level, 2)
            
        if levelsGained > 0:
            if levelsGained > 1:
                engine.log("Level up! You have gained %s levels!" % levelsGained)
            else:
                engine.log("Level up!")

        return levelsGained