import math

import pygame
from pygame import Rect

from enum import *

import content
import engine
import maps

class Camera:
    def __init__(self):
        self.x = 0
        self.y = 0
        self.topLeft = (0, 0)
        self.screenTilesX = 0
        self.screenTilesY = 0
        self.target = None
        
    def update(self, screen):
        if self.target != None:
            self.x = self.target.x
            self.y = self.target.y
        
        self.screenTilesX = int(math.ceil(1.0 * screen.get_width() / engine.tileSize))
        self.screenTilesY = int(math.ceil(1.0 * screen.get_height() / engine.tileSize))
        
        self.topLeft = (self.x - self.screenTilesX / 2, self.y - self.screenTilesY / 2)
    
    def translate(self, x, y):
        return ((x - self.topLeft[0]) * engine.tileSize, (y - self.topLeft[1]) * engine.tileSize)

class Renderable:
    def __init__(self):
        pass
    
    def render(self, screen):
        pass

class Renderer:
    def __init__(self, scale):
        self.scale = scale
        self.buffer = pygame.Surface((engine.screenWidth, engine.screenHeight))
        self.screen = pygame.display.set_mode((self.buffer.get_width() * self.scale, self.buffer.get_height() * self.scale))
        self.renderables = []
        
    def resize(self, scale):
        self.scale = scale
        self.screen = pygame.display.set_mode((self.buffer.get_width() * self.scale, self.buffer.get_height() * self.scale))

    def render(self):
        self.buffer.fill((0, 0, 0))
        for renderable in self.renderables:
            renderable.render(self.buffer)
        pygame.transform.scale(self.buffer, (self.screen.get_width(), self.screen.get_height()), self.screen)
        pygame.display.flip()

class LevelRenderer(Renderable):
    def __init__(self, level):
        Renderable.__init__(self)
        self.level = level # TODO: Make weakref
        self.camera = Camera()
        
    def drawImage(self, screen, x, y, img):
        screen.blit(img, self.camera.translate(x, y))

    def render(self, screen):
        Renderable.render(self, screen)
        
        floor = [content.getImage("floor0"), \
                 content.getImage("floor1"), \
                 content.getImage("floor2"), \
                 content.getImage("floor3")]
        wall = [content.getImage("wall0"), \
                content.getImage("wall1"), \
                content.getImage("wall2"), \
                content.getImage("wall3"), \
                content.getImage("wall4"), \
                content.getImage("wall5"), \
                content.getImage("wall6"), \
                content.getImage("wall7"), \
                content.getImage("wall8"), \
                content.getImage("wall9"), \
                content.getImage("wall10"), \
                content.getImage("wall11")]
        stairsUp = content.getImage("stairsup")
        stairsDown = content.getImage("stairsdown")
       
        self.camera.update(screen)
        
        for y in xrange(self.camera.topLeft[1], self.camera.topLeft[1] + self.camera.screenTilesY):
            for x in xrange(self.camera.topLeft[0], self.camera.topLeft[0] + self.camera.screenTilesX):
                tile = self.level.getTile(x, y)
                if tile != None:
                    if tile.type == TileType.WALL:
                        self.drawImage(screen, x, y, wall[tile.style])
                    elif tile.type == TileType.FLOOR:
                        self.drawImage(screen, x, y, floor[tile.style])
                    elif tile.type == TileType.STAIRSDOWN:
                        self.drawImage(screen, x, y, stairsDown)
                    elif tile.type == TileType.STAIRSUP:
                        self.drawImage(screen, x, y, stairsUp)

        for mob in self.level.mobiles:
            relX = mob.x - self.camera.topLeft[0]
            relY = mob.y - self.camera.topLeft[1]
            
            if relX < 0 or relY < 0 or relX >= self.camera.screenTilesX or relY >= self.camera.screenTilesY:
                continue
            
            self.drawImage(screen, mob.x, mob.y, mob.img)
