import weakref
import pygame
from pygame import Rect

import render
import font
import content
import engine

class UI(render.Renderable):
    def __init__(self, renderer):
        self.skillPane = SkillPane()
        self.possessPane = PossessPane()
        self.statsPane = StatsPane()
        self.bottomPane = BottomPane()
        self.textLog = TextLog()
        self.deathScreen = DeathScreen()
        
        renderer.renderables.append(self.bottomPane)
        renderer.renderables.append(self.textLog)
        renderer.renderables.append(self.skillPane)
        renderer.renderables.append(self.statsPane)
        renderer.renderables.append(self.possessPane)
        renderer.renderables.append(self.deathScreen)
         
class DeathScreen(render.Renderable):
    def __init__(self):
        render.Renderable.__init__(self)    
        self.visible = False
        
    def render(self, screen):
        if self.visible:
            screen.blit(content.getImage("gameover"), (0, 0))
           
class TextLog(render.Renderable):
    def __init__(self):
        render.Renderable.__init__(self)
        self.style = 6
        
    def switchStyle(self):
        self.style += 1
        if self.style > 6:
            self.style = 0
        
    def render(self, screen):
        if self.style == 0:
            return
        
        pane = content.getImage("bottompane")
        bg = content.getImage("textlog")
        
        y = screen.get_height() - pane.get_height()

        if self.style < 6:
            lines = self.style
            
        if self.style == 6:
            screen.blit(bg, (0, 0))
        else:
            height = 4 + 13 * lines
            screen.blit(bg, (0, y - height), Rect(0, 0, bg.get_width(), height))
        
        y -= 15
        i = 0
        for line in engine.logText:
            if self.style < 6 and i == lines:
                break
            font.drawText(screen, line, 3, y, color=(255, 255, 255), background=False)
            y -= 13
            i += 1
       
class BottomPane(render.Renderable):
    def __init__(self):
        render.Renderable.__init__(self)
        self.floor = 1
        
    def render(self, screen):
        pane = content.getImage("bottompane")
        y = screen.get_height() - pane.get_height()
        screen.blit(pane, (0, screen.get_height() - pane.get_height()))
        
        x = 5
        y += 3
        font.drawText(screen, "Lvl: %s   HP: %s/%s   SP: %s/%s      %sF" % 
                      (engine.player.level, engine.player.mobile.health,
                       engine.player.mobile.maxHealth, int(engine.player.sp), engine.player.maxSp,
                       self.floor), x, y, True)
                 
class PossessPane(render.Renderable):
    def __init__(self):
        render.Renderable.__init__(self)
        self.choice = True
        self.visible = False
        self.mob = None
        
    def moveCursor(self, event, key):
        self.choice = not self.choice
        
    def render(self, screen):
        if not self.visible:
            return
        
        pane = content.getImage("possesspane")
        top = (screen.get_height() - pane.get_height()) / 2
        left = 11
        screen.blit(content.getImage("possesspane"), (11, top))
        
        top += 4
        
        font.drawText(screen, "You have defeated %s" % self.mob.getName(), left, top+4, True, centered=True, width=pane.get_width())
        font.drawText(screen, "Possess this vessel?", left, top+18, True, centered = True, width=pane.get_width())
        
        x = left + pane.get_width() / 2
        y = top+46
        font.drawText(screen, "Yes", x - 32, y, True)
        font.drawText(screen, "No", x + 10, y, True)
        
        if self.choice:
            screen.blit(content.getImage("cursor"), (x - 32 - 18, y))
        else:
            screen.blit(content.getImage("cursor"), (x + 10 - 18, y))

class StatsPane(render.Renderable):
    def __init__(self):
        render.Renderable.__init__(self)
        self.cursorPos = 0
        self.visible = False
        
    def moveCursor(self, event, key):
        if key == pygame.K_KP2:
            self.cursorPos += 1
        elif key == pygame.K_KP8:
            self.cursorPos -= 1
        elif key == pygame.K_KP6:
            self.cursorPos += 1
        elif key == pygame.K_KP4:
            self.cursorPos -= 1
            
        while self.cursorPos >= 6:
            self.cursorPos -= 6
        while self.cursorPos < 0:
            self.cursorPos += 6
                
    def render(self, screen):
        if not self.visible:
            return
        
        pane = content.getImage("pane")
        screen.blit(pane, (11, 30))
        
        x = 24
        startY = 34
        y = startY
        i = 0
        
        player = engine.player
        mob = engine.player.mobile
        
        if engine.player.statPoints > 0:
            screen.blit(content.getImage("cursor"), (x - 18, y + 14 * (self.cursorPos + 1)))
        
        font.drawText(screen, "Status", x, y, True); y += 14
        font.drawText(screen, "HP: %s/%s (+%s)" % (mob.health, mob.maxHealth, engine.player.health), x, y, True); y += 14
        font.drawText(screen, "Str: %s (+%s)" % (mob.str, engine.player.str), x, y, True); y += 14
        font.drawText(screen, "Agi: %s (+%s)" % (mob.agi, engine.player.agi), x, y, True); y += 14
        font.drawText(screen, "Con: %s (+%s)" % (mob.con, engine.player.con), x, y, True); y += 14
        font.drawText(screen, "Mag: %s (+%s)" % (mob.mag, engine.player.mag), x, y, True); y += 14
        font.drawText(screen, "Wil: %s (+%s)" % (mob.wil, engine.player.wil), x, y, True); y += 28
       
        font.drawText(screen, "Level: %s" % (player.level), x, y, True); y += 14
        font.drawText(screen, "Exp: %s/%s" % (player.exp, player.expToLevel), x, y, True); y += 14
        font.drawText(screen, "SP: %s/%s" % (player.sp, player.maxSp), x, y, True); y += 14
        font.drawText(screen, "Stat points: %s" % (player.statPoints), x, y, True); y += 14
        
        y = startY
        x += int(round(pane.get_width() * 0.45))
        
        font.drawText(screen, "Current form:", x, y, True); y += 14
        font.drawText(screen, "%s" % (mob.getName()), x, y, True); y += 28
        
        font.drawText(screen, "Unlockable skills:", x, y, True); y += 14
        for skill in mob.skills:
            font.drawText(screen, "%s" % (skill.name), x, y, True)
            y += 14

class SkillPane(render.Renderable):
    def __init__(self):
        render.Renderable.__init__(self)
        self.cursorPos = 0
        self.visible = False
        self.skillsPerColumn = 10
        
    def moveCursor(self, event, key):
        if key == pygame.K_KP2:
            self.cursorPos += 1
        elif key == pygame.K_KP8:
            self.cursorPos -= 1
        elif key == pygame.K_KP6:
            self.cursorPos += self.skillsPerColumn
        elif key == pygame.K_KP4:
            self.cursorPos -= self.skillsPerColumn
            
        while self.cursorPos >= len(engine.player.skills):
            self.cursorPos -= len(engine.player.skills)
        while self.cursorPos < 0:
            self.cursorPos += len(engine.player.skills)
            
    def getSelectedSkill(self):
        return engine.player.skills[self.cursorPos]
     
    def render(self, screen):
        if not self.visible:
            return
        
        screen.blit(content.getImage("pane"), (11, 30))
        
        x = 24
        startY = 34
        y = startY
        i = 0
        
        curSkill = engine.player.skills[self.cursorPos]
        if curSkill.passive:
            info = "Passive skill"
        else:
            info = "Rng: %s, TrgtSelf: %s" % (curSkill.range, curSkill.canTargetSelf)
            if curSkill.isAttack:
                if curSkill.magical:
                    info += ", MagAtk"
                else:
                    info += ", PhysAtk"
                info += ", Dmg: %s%%" % (int(round(curSkill.dmgScalar * 100)))
        
        descY = startY + (self.skillsPerColumn * 14)
        font.drawText(screen, curSkill.desc, x - 6, descY, True)
        font.drawText(screen, info, x - 6, descY + 14, True)
        
        for skill in engine.player.skills:
            if skill.passive:
                font.drawText(screen, "%s" % (skill.name), x, y, True)
            else:
                font.drawText(screen, "%s (%s)" % (skill.name, skill.cost), x, y, True)
                
            if i == self.cursorPos:
                screen.blit(content.getImage("cursor"), (x - 18, y))

            y += 14
            if i != 0 and (i+1) % self.skillsPerColumn == 0:
                x += 97
                y = startY

            i += 1
